# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

import logging
from sweetrpg_model_core.model.base import Model


class Volume(Model):
    """A model object representing an RPG volume (digital or print)."""

    def __init__(self, *args, **kwargs):
        """Creates a new Volume object."""
        logging.debug("args: %s, kwargs: %s", args, kwargs)

        super().__init__(*args, **kwargs)

        self.title = kwargs.get("title")
        self.description = kwargs.get("description")
        self.properties = kwargs.get("properties")
        self.tags = kwargs.get("tags")
        self.system_ids = kwargs.get("system_ids")
        self.publisher_ids = kwargs.get("publisher_ids")
        self.studio_ids = kwargs.get("studio_ids")
        self.license_ids = kwargs.get("license_ids")



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

import logging
from sweetrpg_model_core.model.base import Model


class Review(Model):
    """A model object representing a review for a Volume."""

    def __init__(self, *args, **kwargs):
        """Creates a new Review object.

        :key str title: The title of the review.
        :key str text: The text of the review.
        :key str locale: The locale of the review's contents.
        :key str volume: The volume ID to which this review belongs.
        """
        logging.debug("args: %s, kwargs: %s", args, kwargs)

        super().__init__(*args, **kwargs)

        self.title = kwargs.get("title")
        self.body = kwargs.get("body")
        self.language = kwargs.get("language")
        self.tags = kwargs.get("tags")
        self.volume_id = kwargs.get("volume_id")



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow_jsonapi import fields
from marshmallow_jsonapi.flask import Relationship
from sweetrpg_api_core.schema.base import BaseAPISchema
from sweetrpg_catalog_objects.model.volume import Volume


class VolumeAPISchema(BaseAPISchema):
    model_class = Volume

    class Meta:
        type_ = "volume"
        self_view = "volume_detail"
        self_view_kwargs = {"id": "<id>"}
        self_view_many = "volume_list"

    title = fields.String(required=True)  # , load_only=True)
    description = fields.String(required=True)  # , load_only=True)
    slug = fields.String(required=True)  # , load_only=True)
    properties = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    system_ids = fields.List(fields.String(required=True))  # , load_only=True)
    publisher_ids = fields.List(fields.String(required=True))  # , load_only=True)
    studio_ids = fields.List(fields.String(required=True))  # , load_only=True)
    license_ids = fields.List(fields.String(required=True))  # , load_only=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow import fields
from sweetrpg_catalog_objects.model.contribution import Contribution
from sweetrpg_model_core.schema.base import BaseSchema


class ContributionSchema(BaseSchema):
    model_class = Contribution

    person_id = fields.String(required=True)  # , load_only=True)
    volume_id = fields.String(required=True)
    roles = fields.List(fields.String(required=True))



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

import logging
from sweetrpg_model_core.model.base import Model


class Studio(Model):
    """A model object representing RPG studios."""

    def __init__(self, *args, **kwargs):
        """Creates a new Studio object.

        :key str name: The name of the studio.
        """
        logging.debug("args: %s, kwargs: %s", args, kwargs)

        super().__init__(*args, **kwargs)

        self.name = kwargs.get("name")
        self.website = kwargs.get("website")
        self.notes = kwargs.get("notes")
        self.tags = kwargs.get("tags")
        self.properties = kwargs.get("properties")



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow_jsonapi.flask import Schema, Relationship
from marshmallow_jsonapi import fields
from marshmallow import post_load
from sweetrpg_catalog_objects.model.person import Person
from sweetrpg_catalog_objects.utils.user import add_user_info
from sweetrpg_api_core.schema.base import BaseAPISchema


class PersonAPISchema(BaseAPISchema):
    class Meta:
        type_ = "person"
        self_view = "person_detail"
        self_view_kwargs = {"id": "<id>"}
        self_view_many = "person_list"

    @post_load
    def make_object(self, data, **kwargs):
        data = add_user_info(data)
        return Person(**data)

    id = fields.String()  # as_string=True, dump_only=True)
    name = fields.String(required=True)  # required=True) #, load_only=True)
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    properties = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow_jsonapi.flask import Schema, Relationship
from marshmallow_jsonapi import fields
from sweetrpg_catalog_objects.model.system import System
from sweetrpg_api_core.schema.base import BaseAPISchema


class SystemAPISchema(BaseAPISchema):
    model_class = System

    class Meta:
        type_ = "system"
        self_view = "system_detail"
        self_view_kwargs = {"id": "<id>"}
        self_view_many = "system_list"

    game_system = fields.String(required=True)  # , load_only=True)
    name = fields.String(required=True)
    edition = fields.String(required=True)
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

import logging
from sweetrpg_model_core.model.base import Model


class License(Model):
    """A model object representing an RPG license."""

    def __init__(self, *args, **kwargs):
        """Creates a new License object."""
        logging.debug("args: %s, kwargs: %s", args, kwargs)

        super().__init__(*args, **kwargs)

        self.title = kwargs.get("title")
        self.description = kwargs.get("description")
        self.body = kwargs.get("body")
        self.properties = kwargs.get("properties")
        self.tags = kwargs.get("tags")
        self.volume_ids = kwargs.get("volume_ids")



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow import fields, INCLUDE
from sweetrpg_catalog_objects.model.review import Review
from sweetrpg_model_core.schema.base import BaseSchema


class ReviewSchema(BaseSchema):
    model_class = Review

    title = fields.String(required=True)  # , load_only=True)
    body = fields.String(required=True)  # , load_only=True)
    language = fields.String(required=True)
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    volume = fields.String(required=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""user.py

Utility functions for managing users.
"""


from flask import current_app, session
from sweetrpg_model_core import constants


def add_user_info(data: dict) -> dict:
    """Set user audit fields, such as `created_by` and `updated_by` with current user info.
    If the application is in debug mode, the system user ID is used.

    :param dict data: The data dictionary to update.
    :returns data: The data that was passed in, with user audit fields set.
    """
    if current_app.config['DEBUG']:
        data['updated_by'] = constants.SYSTEM_USER_ID
    elif session['user']:
        data['created_by'] = session['user']['id']

    if not data.get('created_by'):
        data['created_by'] = constants.SYSTEM_USER_ID

    return data

# def has_role(user: User, role_name: str):
#     for role_assoc in user.roles:
#         if role_assoc.enabled and role_assoc.roles.name == role_name:
#             return True

#     return False


# def create_or_add_user(userinfo: dict, role_name: str = model_constants.ROLE_USER):
#     subject = userinfo['sub']

#     # find the role
#     role = Role.query.filter_by(name=role_name).first()
#     if not role:
#         current_app.logger.error(f"Role {role_name} not found!")
#         return None, None

#     # find the user object
#     user = None

#     email = userinfo.get('email')
#     nickname = userinfo.get('nickname')
#     name = userinfo.get('name')

#     if email:
#         user = User.query.filter_by(email=email).first()
#         if not user:
#             user = User(email=email)
#             user.nickname = userinfo['nickname']
#             user.name = userinfo['name']
#             if userinfo.get('picture'):
#                 current_app.logger.info("Using provided picture URL for avatar.")
#                 user.avatar_url = userinfo['picture']
#             else:
#                 current_app.logger.info("Calculating hash for gravatar...")
#                 canonical_email = email.strip().lower()
#                 email_hash = hashlib.md5(canonical_email).hexdigest()
#                 current_app.logger.debug(f"email_hash: {email_hash}")
#                 user.avatar_url = f'https://www.gravatar.com/avatar/{email_hash}?s=30'
#     elif nickname:
#         pass # TODO
#     elif name:
#         pass # TODO
#     else:
#         # TODO
#         raise Exception('invalid user info')

#     if user:
#         db.session.add(user)
#         db.session.commit()

#         role_assoc = UserRole(user, role)
#         db.session.add(role_assoc)
#         db.session.commit()
#         print(user)

#         # find identity object
#         identity = Identity.query.filter_by(subject=subject).first()
#         if not identity:
#             source = subject.split('|', 2)[0]
#             identity = Identity(user=user, source=source, subject=subject)
#             # TODO

#             db.session.add(identity)
#             db.session.commit()
#         print(identity)

#     return user, identity



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow import fields
from sweetrpg_catalog_objects.model.license import License
from sweetrpg_model_core.schema.base import BaseSchema


class LicenseSchema(BaseSchema):
    model_class = License

    title = fields.String(required=True)  # , load_only=True)
    short_title = fields.String(required=True)  # , load_only=True)
    version = fields.String()  # , load_only=True)
    deed = fields.String()  # , load_only=True)
    legal_code = fields.String()  # , load_only=True)
    url = fields.String()  # , load_only=True)
    status = fields.String(required=True)  # , load_only=True)
    availability = fields.String(required=True)  # , load_only=True)
    release_date = fields.Date()  # , load_only=True)
    revocation_date = fields.Date()  # , load_only=True)
    notes = fields.String()  # , load_only=True)
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    properties = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    volume_ids = fields.List(fields.String(required=True))  # , load_only=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from datetime import datetime
from mongoengine import Document, fields
from sweetrpg_catalog_objects.db.embedded.property.document import PropertyDocument
from sweetrpg_catalog_objects.db.embedded.tag.document import TagDocument


class LicenseDocument(Document):
    """A mapping object to convert MongoDB data to a License object."""

    meta = {
        "indexes": [
            {"name": "license_title", "fields": ["title"]},
        ],
        "db_alias": "default",
        "collection": "licenses",
        "strict": False,
    }

    source_id = fields.StringField()

    # basic properties
    title = fields.StringField(required=True)
    short_title = fields.StringField(required=True)
    version = fields.StringField()
    deed = fields.StringField()
    legal_code = fields.StringField()
    url = fields.StringField()
    status = fields.StringField(required=True)
    availability = fields.StringField(required=True)
    release_date = fields.DateField()
    revocation_date = fields.DateField()
    notes = fields.StringField()
    properties = fields.ListField(fields.EmbeddedDocumentField(PropertyDocument))
    tags = fields.ListField(fields.EmbeddedDocumentField(TagDocument))
    volumes = fields.ListField(fields.ReferenceField("VolumeDocument"))

    # audit properties
    created_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    created_by = fields.StringField(default="system", required=True)
    updated_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    updated_by = fields.StringField(default="system", required=True)
    deleted_at = fields.DateTimeField(null=True)
    deleted_by = fields.StringField(default="system", required=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from datetime import datetime
from mongoengine import Document, fields
from pymongo import ASCENDING


class ContributionDocument(Document):
    """A model object representing an RPG contribution."""

    meta = {
        "indexes": [
            # {"name": "contribution_name", "fields": ["name"]},
            {"name": "person_volume_unique", "fields": ["person_id", "volume_id"], "unique": True}
        ],
        "db_alias": "default",
        "collection": "contributions",
        "strict": False,
    }

    source_id = fields.StringField()

    # basic properties
    roles = fields.ListField(fields.StringField(required=True), required=True)

    # relations
    person_id = fields.StringField(min_length=1, max_length=30, required=True)
    volume_id = fields.StringField(min_length=1, max_length=30, required=True)

    # audit properties
    created_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    created_by = fields.StringField(default="system", required=True)
    updated_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    updated_by = fields.StringField(default="system", required=True)
    deleted_at = fields.DateTimeField(null=True)
    deleted_by = fields.StringField(null=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow_jsonapi.flask import Schema, Relationship
from marshmallow_jsonapi import fields
from sweetrpg_catalog_objects.model.studio import Studio
from sweetrpg_api_core.schema.base import BaseAPISchema


class StudioAPISchema(BaseAPISchema):
    model_class = Studio

    class Meta:
        type_ = "studio"
        self_view = "studio_detail"
        self_view_kwargs = {"id": "<id>"}
        self_view_many = "studio_list"

    name = fields.String(required=True)  # , load_only=True)
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    properties = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
